package jetbrains.buildServer.templateGen;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.filters.TokenFilter;

/**
 * @author Yegor.Yarko
 *         Date: 22.08.2010
 */
public class ReplacePreservingCaseFilter implements TokenFilter.Filter {
    private String from;
    private String to;

    /**
     * the from attribute
     *
     * @param from the string to replace
     */
    public void setFrom(String from) {
        this.from = from;
    }

    /**
     * the to attribute
     *
     * @param to the string to replace 'from' with
     */
    public void setTo(String to) {
        this.to = to;
    }

    /**
     * Filter a string 'line' replacing from with to
     * (C&P from the Replace task)
     *
     * @param line the string to be filtered
     * @return the filtered line
     */
    public String filter(String line) {
        String lowercaseLine = line.toLowerCase();
        String lowercaseFrom = from.toLowerCase();
        if (from == null) {
            throw new BuildException("Missing from in stringreplace");
        }
        StringBuffer ret = new StringBuffer();
        int start = 0;
        int found = lowercaseLine.indexOf(lowercaseFrom);
        while (found >= 0) {
            // write everything up to the from
            if (found > start) {
                ret.append(line.substring(start, found));
            }

            // write the replacement to
            if (to != null) {
                if (line.startsWith(from, found)) {
                    ret.append(to);
                }else if (line.startsWith(lowercaseFrom, found)) {
                    ret.append(to.toLowerCase());
                }else{
                    ret.append(line.substring(found, from.length()));
                }
            }

            // search again
            start = found + from.length();
            found = lowercaseLine.indexOf(lowercaseFrom, start);
        }

        // write the remaining characters
        if (line.length() > start) {
            ret.append(line.substring(start, line.length()));
        }

        return ret.toString();
    }
}
